*-----------------------------------------------------------
* Title         : Photo Y2K2 / PGM 3-in-1 protection simulation
* Written by    : IQ_132
* Date          : 2023
* Description   : Simulate the protection routines required by these games in 68K assembly
*-----------------------------------------------------------
    ORG    $2f0000
START:                  ; first instruction of program

debug				equ		0	; 1 to enable debug

COMMAND_WORD		equ		$2f0ff8
DATA_WORD		    equ	    $2f0ffc
SPRITE_DECODE		equ		$280000

RESPONSE_VALUE 		equ		$880000

_unused_ram         equ     $81e000
_unused_ram2        equ     $81e080

_slots			    equ	    _unused_ram	        ; 40 bytes
_response		    equ	    _unused_ram2+$0     ; 4 bytes
_key                equ     _unused_ram2+$4     ; 4 bytes
_slot_select		equ	    _unused_ram2+$8     ; 2 bytes
_sim_value		    equ	    _unused_ram2+$a		; 2 bytes

_background_select	equ		_unused_ram2+$c		; 2 bytes
_text_select		equ		_unused_ram2+$e		; 2 bytes
_ba_value			equ		_unused_ram2+$10	; 2 bytes
_sprite_value		equ		_unused_ram2+$12	; 2 bytes
_sprite_pos			equ		_unused_ram2+$14	; 4 bytes
_y_value			equ		_unused_ram2+$18	; 2 bytes

_protection_routine
    movem.l     D0-D2/A0, -(A7)

	movea.l		COMMAND_WORD, A0
	move.w		(A0), D0

	tst.b		enable_debug
	beq.s		_no_debug_command
		movea.l		DATA_WORD, A0
		move.w		(A0), D1
		move.w		D1, $500000
		move.w		D0, $500002 ; command
_no_debug_command

    move.w      D0, D1
    lsr.w       #$8, D1
    cmp.b       #$ff, D1
    bne.s       _skip_calc_key_ff
    	move.l      #$ffffffff, _key
_skip_calc_key_ff
    move.w      _key, D1

    ; apply key to command
    eor.b       D1, D0
    andi.w      #$ff, D0

    ; apply key to sim value
	movea.l		DATA_WORD, A0
	move.w		(A0), D2
	eor.w       D1, D2
	move.w      D2, _sim_value

	tst.b		enable_debug
	beq.s		_no_debug_commandvalue
		move.w		D0, $500012 ; hack command
		move.w		D2, $500010 ; hack value
_no_debug_commandvalue

	move.l		#RESPONSE_VALUE, _response

	cmp.b		#$30, D0
	beq			_30_routine
	cmp.b		#$32, D0
	beq			_32_routine
    cmp.b		#$3a, D0
	beq			_3a_routine
	cmp.b		#$40, D0
	beq			_40_routine
    cmp.b		#$4d, D0
	beq			_4d_routine
    cmp.b		#$50, D0
	beq			_50_routine
    cmp.b		#$5e, D0
	beq			_5e_routine
    cmp.b		#$60, D0
	beq			_60_routine
    cmp.b		#$6a, D0
	beq			_6a_routine
    cmp.b		#$70, D0
	beq			_70_routine
    cmp.b		#$7b, D0
	beq			_7b_routine
    cmp.b		#$80, D0
	beq			_80_routine
    cmp.b		#$8c, D0
	beq			_8c_routine
	cmp.b		#$99, D0
	beq			_99_routine
    cmp.b		#$9d, D0
	beq			_9d_routine
    cmp.b		#$a0, D0
	beq			_a0_routine
    cmp.b		#$ae, D0
	beq			_ae_routine
    cmp.b		#$b0, D0
	beq			_b0_routine
    cmp.b		#$ba, D0
	beq			_ba_routine
	cmp.b		#$c0, D0
	beq			_c0_routine
    cmp.b		#$c1, D0
	beq			_c1_routine
    cmp.b		#$c3, D0
	beq			_c3_routine
    cmp.b		#$c5, D0
	beq			_c5_routine
    cmp.b		#$c7, D0
	beq			_c7_routine
    cmp.b		#$cc, D0
	beq			_cc_routine
    cmp.b		#$cf, D0
	beq			_cf_routine
    cmp.b		#$d0, D0
	beq			_d0_routine
    cmp.b		#$d6, D0
	beq			_d6_routine
    cmp.b		#$dc, D0
	beq			_dc_routine
    cmp.b		#$e0, D0
	beq			_e0_routine
    cmp.b		#$e5, D0
	beq			_e5_routine
    cmp.b		#$e7, D0
	beq			_e7_routine
    cmp.b		#$f8, D0
	beq			_f8_routine

	; fall through if not found...
_end_protection_routine
    moveq.l     #$0, D0    ; update key!
    moveq.l     #$0, D1
    move.b      _key, D0
    add.w       #$1, D0
    andi.w      #$ff, D0
    cmpi.b      #$ff, D0
    bne.s       _skip_reset_key
    moveq.l     #$1, D0
_skip_reset_key
    move.b      D0, D1
    lsl.w       #$8, D0
    move.b      D1, D0
    move.w      D0, D1
    swap        D0
    move.w      D1, D0
    move.l      D0, _key
    move.l      _response, D1 ; apply key to response!

	tst.b		enable_debug
	beq.s		_no_debug_key_response
		move.w		D0, $500014 ; key
		move.w		D1, $500016 ; value low
		swap		D1
		move.w		D1, $500018 ; value high
		swap		D1
_no_debug_key_response

    eor.l       D0, D1
    move.l      D1, _response
    movem.l     (A7)+, D0-D2/A0
	rts



_3a_routine
	move.w		_sim_value, D0
	andi.w		#$f, D0
	lsl.w		#$2, D0
	lea.l		_slots, A0
	move.l		#$0, (A0,D0.w)
	bra         _end_protection_routine


_99_routine
    move.l      #$1, _key
	bra	       _end_protection_routine


_c0_routine
	move.w		_sim_value, _text_select
	bra			_end_protection_routine


_c1_routine
	moveq.l		#$0, D0
	moveq.l		#$0, D1

	move.w		_ba_value, D0
	move.w		D0, D1
	andi.w		#$f000, D1
	cmpi.w		#$f000, D1
	bne.s		_not_y_f000
		move.w	_sim_value, D1
		neg.w	D1
		cmp.w	D0, D1
		bge.s	_c1_y_set
			move.w	D0, D1
			bra.s	_c1_y_set
_not_y_f000
	tst.w		D0
	bne.s		_y_not_0
		move.w	D0, D1
		bra.s	_c1_y_set
_y_not_0
		move.w	_sim_value, D1
		cmp.w	D1, D0
		bge.s	_c1_y_set
			move.w	D0, D1
_c1_y_set
	move.w		D1, _y_value

	move.w		_sprite_value, D0
	move.w		D0, D1
	andi.w		#$f000, D1
	cmpi.w		#$f000, D1
	bne.s		_not_x_f000
		move.w	_sim_value, D1
		neg.w	D1
		cmp.w	D0, D1
		bge.s	_c1_x_set
			move.w	D0, D1
			bra.s	_c1_x_set
_not_x_f000
	tst.w	D0
	bne.s	_x_not_0
		move.w	D0, D1
		bra.s	_c1_x_set
_x_not_0
	move.w	_sim_value, D1
	cmp.w	D1, D0
	bge.s	_c1_x_set
				move.w	D0, D1
_c1_x_set

	moveq.l		#$0, D0
	move.w		_y_value, D0
	lsl.l		#$8, D0
	or.w		D1, D0
	move.l		D0, _response
	bra 		_end_protection_routine


_c3_routine
	move.l		#$904000, D0
	moveq.l		#$0, D1
	move.w		_sim_value, D1
	lsl.l		#$8, D1
	add.l		D1, D0
	moveq.l		#$0, D1
	move.w		_text_select, D1
	lsl.l		#$2, D1
	add.l		D1, D0
	move.l		D0, _response
	bra			_end_protection_routine


_c5_routine
	move.w		_sim_value, D0
	andi.w		#$f, D0
	lsl.w		#$2, D0
	lea.l		_slots, A0
	move.l		(A0,D0.w), D1
	subq.l		#$1, D1
	move.l		D1, (A0,D0.w)
	bra         _end_protection_routine

_c7_routine
	move.w		_sim_value, _sprite_value
	bra         _end_protection_routine


_cb_routine
	move.w		_sim_value, _background_select
	bra			_end_protection_routine


_cc_routine
	move.l		#$900000, D0
	move.w		_sim_value, D0
	andi.w		#$f, D0
	lsl.w		#$8, D0
	move.w		_background_select, D1
	lsl.w		#$2, D1
	add.w		D1, D0
	move.l		D0, _response
	bra			_end_protection_routine



_cf_routine
	move.w		_sim_value, _ba_value
	bra         _end_protection_routine


_d0_routine
	move.l		#$a01000, D0
	moveq.l		#$0, D1
	move.w		_sim_value, D1
	lsl.w		#$5, D1
	add.w		D1, D0
	move.l		D0, _response
	bra			_end_protection_routine


_d6_routine
	move.w		_sim_value, D0
	andi.w		#$f, D0
	lsl.w		#$2, D0
	lea.l		_slots, A0
	move.l		(A0,D0.w), D1
	addq.l		#$1, D1
	move.l		D1, (A0,D0.w)
	bra         _end_protection_routine


_dc_routine
	move.l		#$a00800, D0
	moveq.l		#$0, D1
	move.w		_sim_value, D1
	lsl.w		#$6, D1
	add.w		D1, D0
	move.l		D0, _response
	bra			_end_protection_routine


_e0_routine
	move.l		#$a00000, D0
	move.w		_sim_value, D0
	andi.w		#$1f, D0
	lsl.w		#$6, D0
	move.l		D0, _response
	bra	        _end_protection_routine


_e5_routine
	move.w		_slot_select, D0
	lea.l		_slots, A0
	move.l		(A0,D0.w), D1
	move.w		_sim_value, D1
	move.l		D1, (A0,D0.w)
	bra         _end_protection_routine


_e7_routine
	move.w		_sim_value, D0
	andi.w      #$f000, D0
	rol.w		#$6, D0
	move.w		D0, _slot_select
	lea.l		_slots, A0
	move.l		(A0,D0.w), D1
	swap		D1
	move.w		_sim_value, D1
	andi.w		#$ff, D1
	swap		D1
	move.l		D1, (A0,D0.w)
	bra         _end_protection_routine


_f8_routine
	move.w		_sim_value, D0
	andi.w		#$f, D0
	lsl.w		#$2, D0
	lea.l		_slots, A0
	move.l		(A0,D0.w), _response
	bra         _end_protection_routine


_32_routine
	moveq.l		#$0, D0
	move.w		_sim_value, D0
	lsl.l		#$4, D0
	move.l		D0, _sprite_pos
	; fall through to _30_routine

_30_routine
	move.l		_sprite_pos, D0
	lsl.l		#$1, D0
	lea.l		SPRITE_DECODE, A0
	add.l		D0, A0
	moveq.l		#$0, D1
	move.w		(A0), D1
	move.l		D1, _response

	lsr.l		#$1, D0
	; increment sprite_pos
	move.b		D0, D1
	andi.b		#$f0, D0
	addq.b		#$1, D1
	andi.b		#$f, D1
	or.b		D1, D0
	move.l		D0, _sprite_pos
	bra			_end_protection_routine


_40_routine
	lea.l		py2k2_40_table, A0
	moveq.l		#$0, D0
	move.w		_sim_value, D0
	andi.l		#$7, D0
	lsl.w		#$1, D0
	move.w		(A0,D0.w), D0
	move.l		D0, _response
	bra			_end_protection_routine


_4d_routine
	lea.l		py2k2_4d_table, A0
	moveq.l		#$0, D0
	move.w		_sim_value, D0
	andi.l		#$f, D0
	lsl.w		#$1, D0
	move.w		(A0,D0.w), D0
	move.l		D0, _response
	bra			_end_protection_routine


_50_routine
	lea.l		py2k2_50_table, A0
	moveq.l		#$0, D0
	move.w		_sim_value, D0
	andi.l		#$f, D0
	lsl.w		#$1, D0
	move.w		(A0,D0.w), D0
	move.l		D0, _response
	bra			_end_protection_routine


_5e_routine
	lea.l		py2k2_5e_table, A0
	moveq.l		#$0, D0
	move.w		_sim_value, D0
	andi.l		#$f, D0
	lsl.w		#$1, D0
	move.w		(A0,D0.w), D0
	move.l		D0, _response
	bra			_end_protection_routine


_60_routine
	lea.l		py2k2_60_table, A0
	moveq.l		#$0, D0
	move.w		_sim_value, D0
	andi.l		#$f, D0
	lsl.w		#$1, D0
	move.w		(A0,D0.w), D0
	move.l		D0, _response
	bra			_end_protection_routine


_6a_routine
	lea.l		py2k2_6a_table, A0
	moveq.l		#$0, D0
	move.w		_sim_value, D0
	andi.l		#$f, D0
	lsl.w		#$1, D0
	move.w		(A0,D0.w), D0
	move.l		D0, _response
	bra			_end_protection_routine


_70_routine
	lea.l		py2k2_70_table, A0
	moveq.l		#$0, D0
	move.w		_sim_value, D0
	andi.l		#$f, D0
	lsl.w		#$1, D0
	move.w		(A0,D0.w), D0
	move.l		D0, _response
	bra			_end_protection_routine


_7b_routine
	lea.l		py2k2_7b_table, A0
	moveq.l		#$0, D0
	move.w		_sim_value, D0
	andi.l		#$f, D0
	lsl.w		#$1, D0
	move.w		(A0,D0.w), D0
	move.l		D0, _response
	bra			_end_protection_routine


_80_routine
	lea.l		py2k2_80_table, A0
	moveq.l		#$0, D0
	move.w		_sim_value, D0
	andi.l		#$f, D0
	lsl.w		#$1, D0
	move.w		(A0,D0.w), D0
	move.l		D0, _response
	bra			_end_protection_routine


_8c_routine
	lea.l		py2k2_8c_table, A0
	moveq.l		#$0, D0
	move.w		_sim_value, D0
	andi.l		#$f, D0
	lsl.w		#$1, D0
	move.w		(A0,D0.w), D0
	move.l		D0, _response
	bra			_end_protection_routine


_9d_routine
	lea.l		py2k2_9d_table, A0
	moveq.l		#$0, D0
	move.w		_sim_value, D0
	andi.l		#$3, D0
	lsl.w		#$1, D0
	move.w		(A0,D0.w), D0
	move.l		D0, _response
	bra			_end_protection_routine


_a0_routine
	lea.l		py2k2_a0_table, A0
	moveq.l		#$0, D0
	move.w		_sim_value, D0
	andi.l		#$f, D0
	lsl.w		#$1, D0
	move.w		(A0,D0.w), D0
	move.l		D0, _response
	bra			_end_protection_routine


_ae_routine
	lea.l		py2k2_ae_table, A0
	moveq.l		#$0, D0
	move.w		_sim_value, D0
	andi.l		#$f, D0
	lsl.w		#$1, D0
	move.w		(A0,D0.w), D0
	move.l		D0, _response
	bra			_end_protection_routine


_b0_routine
	lea.l		py2k2_b0_table, A0
	moveq.l		#$0, D0
	move.w		_sim_value, D0
	andi.l		#$f, D0
	lsl.w		#$1, D0
	move.w		(A0,D0.w), D0
	move.l		D0, _response
	bra			_end_protection_routine


_ba_routine
	lea.l		py2k2_ba_table, A0
	moveq.l		#$0, D0
	move.w		_sim_value, D0
	andi.l		#$3f, D0
	lsl.w		#$1, D0
	move.w		(A0,D0.w), D0
	move.l		D0, _response
	bra			_end_protection_routine


py2k2_40_table ; 8
	dc.w	$00e0, $00a8, $0080, $0080, $0100, $0080, $0180, $0080 

py2k2_4d_table ; 16
	dc.w	$0002, $0003, $0004, $0005, $0006, $0007, $0008, $0009, $000a, $000b, $000c, $0000, $0001, $000d, $000e, $0000

py2k2_50_table ; 16
	dc.w	$006c, $00a8, $0000, $0000, $0154, $0044, $0000, $0000, $006c, $0044, $0000, $0000, $0000, $0000, $0000, $0000

py2k2_5e_table ; 16
	dc.w	$0000, $0000, $0000, $0000, $01a0, $0064, $0000, $0008, $0020, $0064, $0000, $0008, $0000, $0000, $0000, $0000

py2k2_60_table ; 16
	dc.w	$0064, $00c0, $0000, $0000, $01a0, $003c, $0000, $0000, $2000, $3c00, $0000, $0000, $0000, $0000, $0000, $0000

py2k2_6a_table ; 16
	dc.w	$00e0, $008d, $0012, $0000, $0160, $0069, $0000, $000a, $005c, $0069, $0000, $000a, $0000, $0000, $0000, $0000

py2k2_70_table ; 16
	dc.w	$00e0, $009c, $004a, $0000, $014a, $0069, $0000, $0020, $0076, $0069, $0000, $0020, $0000, $0000, $0000, $0000

py2k2_7b_table ; 16
	dc.w	$00e3, $00b6, $0012, $0000, $01a0, $0064, $0000, $0006, $0020, $0064, $0000, $0006, $0000, $0000, $0000, $0000

py2k2_80_table ; 16
	dc.w	$0002, $0003, $0004, $0005, $0006, $0007, $0008, $0009, $000a, $000b, $000c, $0000, $0001, $000d, $000e, $0000

py2k2_8c_table ; 16
	dc.w	$0000, $0000, $0000, $0000, $019a, $0079, $0000, $000a, $00e2, $003c, $0016, $0000, $002b, $0079, $0000, $000a

py2k2_9d_table ; 4
	dc.w	$00e0, $00a0, $0000, $0000

py2k2_a0_table ; 16
	dc.w	$0020, $00C5, $000C, $0000, $01AC, $00C5, $FFF4, $0000, $01AC, $0046, $FFF4, $0000, $0020, $0046, $000C, $0000

py2k2_ae_table ; 16
	dc.w	$00e2, $008e, $0010, $0000, $014d, $0079, $0010, $0000, $00e2, $0065, $0010, $0000, $0078, $0079, $0010, $0000

py2k2_b0_table ; 16
	dc.w	$00e2, $00b6, $0016, $0000, $019a, $0079, $0000, $0006, $00e2, $003c, $0016, $0000, $002b, $0079, $0000, $0006

py2k2_ba_table ; 64
	dc.w	$002a, $0028, $0026, $0024, $0022, $001e, $001c, $001a, $0016, $0026, $0024, $0022, $0020, $001e, $001a, $0018
	dc.w	$0016, $0014, $0022, $0021, $0020, $001e, $001c, $001a, $0016, $0014, $0014, $001e, $001e, $001e, $001c, $001a
	dc.w	$0018, $0016, $0014, $0014, $001e, $001e, $001c, $001a, $0018, $0016, $0014, $0014, $0014, $0000, $000a, $0014
	dc.w	$0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000

enable_debug
	dc.b	debug

    END    START        ; last line of source

*~Font name~Courier New~
*~Font size~18~
*~Tab type~1~
*~Tab size~4~
